part of 'ship.dart';

class _ShipImpl implements Ship {
  @override
  List<Widget> pageStack = <Widget>[];

  @override
  GlobalKey<NavigatorState> navigationKey = GlobalKey<NavigatorState>();

  @override
  void to({Widget page}) {
    pageStack.add(page);
    navigationKey.currentState.push(RouteUtils.createPageRoute(page));
  }

  @override
  void back({dynamic result}) {
    pageStack.removeLast();
    navigationKey.currentState.pop(result);
  }

  @override
  void replace({Widget page}) {
    pageStack.removeLast();
    pageStack.add(page);

    navigationKey.currentState
        .pushReplacement(RouteUtils.createPageRoute(page));
  }

  @override
  void goBackTo({Widget page}) {
    while (pageStack.last != page) {
      pageStack.removeLast();

      navigationKey.currentState.pop();
    }
  }

  @override
  void closeDialog({result}) {
    navigationKey.currentState.pop(result);
  }

  @override
  Future<T> startPageForResult<T>({Widget page}) async {
    pageStack.add(page);

    var result =
        await navigationKey.currentState.push(RouteUtils.createPageRoute(page));

    if (result.runtimeType != T) {
      throw FloativeException("The received result type: ${result.runtimeType}"
          " does not match the expected type of $T");
    } else {
      return result;
    }
  }

  @override
  Future<dynamic> showBottomSheet(
      {Widget bottomSheet,
      bool isDismissible = true,
      bool enableDrag = true,
      bool isScrollControlled = false,
      Color backGroundColor = Colors.transparent}) async {
    var result = await navigationKey.currentState.push(
        RouteUtils.createBottomSheetRoute(
            isDismissible: isDismissible,
            enableDrag: enableDrag,
            isScrollControlled: isScrollControlled,
            backGroundColor: backGroundColor,
            bottomSheet: bottomSheet));

    return result;
  }

  @override
  Future<dynamic> showDialog({
    Widget dialog,
    bool isDismissible = true,
    Color backGroundColor = const Color(0x80000000),
  }) async {
    var result =
        await navigationKey.currentState.push(RouteUtils.createDialogRoute(
            isDismissible: isDismissible,
            backGroundColor: backGroundColor,
            dialog: Dialog(
              child: dialog,
            )));

    return result;
  }
}
